@@ -183,6 +183,7 @@ vis_identifier_multi_cor <- function(dataset, ids, samples = NULL,
 #' - The second column refers to values indicated in axis Y.
 #' - The third column refers to groups indicated in axis X.
 #' - The fourth column is optional, which indicates facet variable.
+#' @param pairwise.comparisons whether pairwise comparison
 #' @param ... other parameters passing to [ggstatsplot::ggbetweenstats] or [ggstatsplot::ggwithinstats].
 #' @export
 #' @return a (gg)plot object.
@@ -255,12 +256,13 @@ vis_identifier_grp_comparison <- function(dataset = NULL, id = NULL, grp_df, sam
 
   if (do_grp) {
     fun <- if (fun_type == "betweenstats") ggstatsplot::grouped_ggbetweenstats else ggstatsplot::grouped_ggwithinstats
-
+    # colnames(df)[4] = "grouping.var"
     p <- fun(
       data = df,
       x = !!colnames(df)[3],
       y = !!colnames(df)[2],
-      grouping.var = !!colnames(df)[4],
+      # grouping.var = grouping.var,
+      grouping.var = !!rlang::sym(colnames(df)[4]),
       type  = type,
       pairwise.comparisons = pairwise.comparisons,
       p.adjust.method = p.adjust.method,
